# Interactive Message and Joining Broadcasting

### Preparations
Whether you want to send messages or join the broadcasting, you need to set event and message listening before creating and joining a room. Example:
```
TILLiveManager *manager = [TILLiveManager getInstance];
[manager setAVListener:self]; //Set audio/video event listening
[manager setIMListener:self]; //Set message listening
```

### 1. Send text messages

| API | Description |
|---|---|
| sendTextMessage:  succ:  failed: | Send text messages |

| Parameter Type | Parameter Name | Description |
|---|---|---|
| ILVLiveTextMessage | msg | Text message type |
| TCIVoidBlock | succ | "Message sent successfully" callback |
| TCIErrorBlock | failed | "Failed to send message" callback |

* Example:
```
// 1. Send text messages
TILLiveManager *manager = [TILLiveManager getInstance];
ILVLiveTextMessage *msg = [[ILVLiveTextMessage alloc] init];

msg.type = ILVLIVE_IMTYPE_GROUP;    //Broadcast messages (or C2C messages)
msg.text = @"Message content ";        // message content
msg.sendId = @"ID of message sender";
msg.sendId = @"ID of message receiver";

[manager sendTextMessage:msg succ:^{
    NSLog(@"Sent successfully");
} failed:^(NSString *moudle, int errId, NSString *errMsg) {
    NSLog(@"Sending failed");
}];
```

```
// 2. Receive text messages (text messages is received in text message callback)
- (void)onTextMessage:(ILVLiveTextMessage *)msg
{
    NSLog(@"Message received: %@", msg.text);
}
```
### 2. Join broadcasting
#### 2.1 Flowchart of invitation from VJ to viewer for joint broadcasting:
![](http://mc.qcloudimg.com/static/img/ccbafe376da2e175ff41bd681856581e/image.png)
------
#### 2.2 Flowchart of viewer's request for joining the broadcasting:
![](http://mc.qcloudimg.com/static/img/4d21a6ce428740fa16ebc58a0675b3e7/image.png)
------
#### 2.3 APIs
##### (1) API for sending custom messages

| API | Description |
|---|---|
| sendCustomMessage:  succ:  failed: | Send custom messages |

| Parameter Type | Parameter Name | Description |
|---|---|---|
| ILVLiveCustomMessage | msg | Custom message body |
| TCIVoidBlock | succ | "Custom message sent successfully" callback |
| TCIErrorBlock | failed | "Failed to send custom message" callback |

* Example:
```
TILLiveManager *manager = [TILLiveManager getInstance];
ILVLiveCustomMessage *msg = [[ILVLiveCustomMessage alloc] init];
msg.cmd = ILVLIVE_IMCMD_INVITE;     //Invitation message
msg.type = ILVLIVE_IMTYPE_C2C;      //C2C message type
msg.recvId = @"ID of message receiver";

[manager sendCustomMessage:msg succ:^{
    NSLog(@"Invited successfully");
} failed:^(NSString *moudle, int errId, NSString *errMsg) {
    NSLog(@"Invitation failed"); 
}];
```
##### (2) API for joining broadcasting

| API | Description |
|---|---|
| upToVideoMember:  role:  succ:  failed: | Viewer joins the broadcasting, including enabling camera and microphone, switching role and permission, etc. |

| Parameter Type | Parameter Name | Description |
|---|---|---|
| uint64_t | auth | Chatting permission bit. The definitions of all the permission bits can be found in file QAVCommon.h. |
| NSString | role | Role string (generated by the console of App at business side)
| TCIVoidBlock | succ | "Joined the broadcasting successfully" callback |
| TCIErrorBlock | failed | "Failed to join the broadcasting" callback |

* Example:
```
//Viewer can receive the custom message sent by the VJ in the message callback.
- (void)onCustomMessage:(ILVLiveCustomMessage *)msg
{
    TILLiveManager *manager = [TILLiveManager getInstance];
    switch (msg.cmd) 
    {
        case ILVLIVE_IMCMD_INVITE:
        {
            //Receive an invitation and call the API to join the broadcasting
            [manager upToVideoMember:ILVLIVEAUTH_INTERACT role:@"Role configured at the Tencent Cloud backend" succ:^{
                NSLog(@"Joined the broadcasting successfully"); 
            } failed:^(NSString *moudle, int errId, NSString *errMsg) {
                NSLog(@"Failed to join the broadcasting"); 
            }];
        }
        break;
    }
}
```
##### (3) API for setting the area for rendering for joining broadcasting

| API | Description |
|---|---|
| addAVRenderView:  forKey: | Add the area for rendering and set the key for the rendered view (generally, the user ID for the rendered view is used) |

| Parameter Type | Parameter Name | Description |
|---|---|---|
| CGRect | frame | View rendering area |
| NSString | key | View key, which is used as business logic (generally, the user ID for the view is used) |

* Example:
```
- (void)onUserUpdateInfo:(ILVLiveAVEvent)event users:(NSArray *)users
{
    TILLiveManager *manager = [TILLiveManager getInstance];
    switch (event) 
    {
        case ILVLIVE_AVEVENT_CAMERA_ON:
        {
            for (NSString *user in users) 
            {
                //The rendering position of VJ has been specified at the time of creating or joining the room, and needs not to be specified here.
                //You can also specify the rendering position of VJ based on your own logic.
                if(![user isEqualToString:self.host])
                { 
                    [manager addAVRenderView:CGRectMake(20, 20, 120, 160) forKey:user];
                }
            }	
        }
        break;
    }
}
```
If the above steps are correctly performed, the VJ can successfully invite the viewer to the broadcasting. When the viewer joins the broadcasting, multi-stream views appear in the room to show the interactions between VJ and viewers.

